@@ -10,6 +10,8 @@ for signing and checking OAuth 1.0 RFC 5849 requests.
 """
 
 import logging
+log = logging.getLogger("oauthlib")
+
 import sys
 import time
 try:
@@ -95,6 +97,13 @@ class Client(object):
 
     def get_oauth_signature(self, request):
         """Get an OAuth signature to be used in signing a request
+
+        To satisfy `section 3.4.1.2`_ item 2, if the request argument's
+        headers dict attribute contains a Host item, its value will
+        replace any netloc part of the request argument's uri attribute
+        value.
+
+        .. _`section 3.4.1.2`: http://tools.ietf.org/html/rfc5849#section-3.4.1.2
         """
         if self.signature_method == SIGNATURE_PLAINTEXT:
             # fast-path
@@ -107,17 +116,18 @@ class Client(object):
             uri_query=urlparse.urlparse(uri).query,
             body=body,
             headers=headers)
-        logging.debug("Collected params: {0}".format(collected_params))
+        log.debug("Collected params: {0}".format(collected_params))
 
         normalized_params = signature.normalize_parameters(collected_params)
-        normalized_uri = signature.normalize_base_string_uri(request.uri)
-        logging.debug("Normalized params: {0}".format(normalized_params))
-        logging.debug("Normalized URI: {0}".format(normalized_uri))
+        normalized_uri = signature.normalize_base_string_uri(uri,
+            headers.get('Host', None))
+        log.debug("Normalized params: {0}".format(normalized_params))
+        log.debug("Normalized URI: {0}".format(normalized_uri))
 
         base_string = signature.construct_base_string(request.http_method,
             normalized_uri, normalized_params)
 
-        logging.debug("Base signing string: {0}".format(base_string))
+        log.debug("Base signing string: {0}".format(base_string))
 
         if self.signature_method == SIGNATURE_HMAC:
             sig = signature.sign_hmac_sha1(base_string, self.client_secret,
@@ -128,7 +138,7 @@ class Client(object):
             sig = signature.sign_plaintext(self.client_secret,
                 self.resource_owner_secret)
 
-        logging.debug("Signature: {0}".format(sig))
+        log.debug("Signature: {0}".format(sig))
         return sig
 
     def get_oauth_params(self):
@@ -276,13 +286,18 @@ class Client(object):
                 realm=(realm or self.realm))
 
         if self.decoding:
-            logging.debug('Encoding URI, headers and body to %s.', self.decoding)
+            log.debug('Encoding URI, headers and body to %s.', self.decoding)
             uri = uri.encode(self.decoding)
             body = body.encode(self.decoding) if body else body
             new_headers = {}
             for k, v in headers.items():
                 new_headers[k.encode(self.decoding)] = v.encode(self.decoding)
             headers = new_headers
+
+        log.debug('OAuthlib successfully signed the request.')
+        log.debug('URI: %s', uri)
+        log.debug('Headers: %s', headers)
+        log.debug('Body: %s', body)
         return uri, headers, body
 
 
@@ -311,13 +326,13 @@ class Server(object):
     overloading the methods a few properties can be used to configure these
     methods.
 
-    @ safe_characters -> (character set)
-    @ client_key_length -> (min, max)
-    @ request_token_length -> (min, max)
-    @ access_token_length -> (min, max)
-    @ nonce_length -> (min, max)
-    @ verifier_length -> (min, max)
-    @ realms -> [list, of, realms]
+    * @safe_characters -> (character set)
+    * @client_key_length -> (min, max)
+    * @request_token_length -> (min, max)
+    * @access_token_length -> (min, max)
+    * @nonce_length -> (min, max)
+    * @verifier_length -> (min, max)
+    * @realms -> [list, of, realms]
 
     Methods used to validate input parameters. These checks usually hit either
     persistent or temporary storage such as databases or the filesystem. See
@@ -350,9 +365,16 @@ class Server(object):
     equivalent to the running time when given a valid client/resource owner.
     The following properties must be implemented:
 
-    @ dummy_client
-    @ dummy_request_token
-    @ dummy_access_token
+    * @dummy_client
+    * @dummy_request_token
+    * @dummy_access_token
+
+    Example implementations have been provided, note that the database used is
+    a simple dictionary and serves only an illustrative purpose. Use whichever
+    database suits your project and how to access it is entirely up to you.
+    The methods are introduced in an order which should make understanding
+    their use more straightforward and as such it could be worth reading what
+    follows in chronological order.
 
     .. _`whitelisting or blacklisting`: http://www.schneier.com/blog/archives/2011/01/whitelisting_vs.html
     """
@@ -450,7 +472,20 @@ class Server(object):
 
         This method must allow the use of a dummy client_key value.
         Fetching the secret using the dummy key must take the same amount of
-        time as fetching a secret for a valid client.
+        time as fetching a secret for a valid client::
+
+            # Unlikely to be near constant time as it uses two database
+            # lookups for a valid client, and only one for an invalid.
+            from your_datastore import ClientSecret
+            if ClientSecret.has(client_key):
+                return ClientSecret.get(client_key)
+            else:
+                return 'dummy'
+
+            # Aim to mimic number of latency inducing operations no matter
+            # whether the client is valid or not.
+            from your_datastore import ClientSecret
+            return ClientSecret.get(client_key, 'dummy')
 
         Note that the returned key must be in plaintext.
         """
@@ -475,7 +510,20 @@ class Server(object):
         """Retrieves the shared secret associated with the request token.
 
         This method must allow the use of a dummy values and the running time
-        must be roughly equivalent to that of the running time of valid values.
+        must be roughly equivalent to that of the running time of valid values::
+
+            # Unlikely to be near constant time as it uses two database
+            # lookups for a valid client, and only one for an invalid.
+            from your_datastore import RequestTokenSecret
+            if RequestTokenSecret.has(client_key):
+                return RequestTokenSecret.get((client_key, request_token))
+            else:
+                return 'dummy'
+
+            # Aim to mimic number of latency inducing operations no matter
+            # whether the client is valid or not.
+            from your_datastore import RequestTokenSecret
+            return ClientSecret.get((client_key, request_token), 'dummy')
 
         Note that the returned key must be in plaintext.
         """
@@ -485,7 +533,20 @@ class Server(object):
         """Retrieves the shared secret associated with the access token.
 
         This method must allow the use of a dummy values and the running time
-        must be roughly equivalent to that of the running time of valid values.
+        must be roughly equivalent to that of the running time of valid values::
+
+            # Unlikely to be near constant time as it uses two database
+            # lookups for a valid client, and only one for an invalid.
+            from your_datastore import AccessTokenSecret
+            if AccessTokenSecret.has(client_key):
+                return AccessTokenSecret.get((client_key, request_token))
+            else:
+                return 'dummy'
+
+            # Aim to mimic number of latency inducing operations no matter
+            # whether the client is valid or not.
+            from your_datastore import AccessTokenSecret
+            return ClientSecret.get((client_key, request_token), 'dummy')
 
         Note that the returned key must be in plaintext.
         """
@@ -515,14 +576,15 @@ class Server(object):
         """Retrieves a previously stored client provided RSA key.
 
         This method must allow the use of a dummy client_key value. Fetching
-        the rsa key using the dummy key must take the same aount of time
-        as fetching a key for a valid client.
+        the rsa key using the dummy key must take the same amount of time
+        as fetching a key for a valid client. The dummy key must also be of
+        the same bit length as client keys.
 
         Note that the key must be returned in plaintext.
         """
         raise NotImplementedError("Subclasses must implement this function.")
 
-    def get_signature_type_and_params(self, request):
+    def _get_signature_type_and_params(self, request):
         """Extracts parameters from query, headers and body. Signature type
         is set to the source in which parameters were found.
         """
@@ -563,16 +625,22 @@ class Server(object):
         Note that if the dummy client is supplied it should validate in same
         or nearly the same amount of time as a valid one.
 
-        Bad:
+        Ensure latency inducing tasks are mimiced even for dummy clients.
+        For example, use::
 
-            if client_key == self.dummy_client:
+            from your_datastore import Client
+            try:
+                return Client.exists(client_key, access_token)
+            except DoesNotExist:
+                return False
+
+        Rather than::
+
+            from your_datastore import Client
+            if access_token == self.dummy_access_token:
                 return False
             else:
-                return storage.has_client(client_key)
-
-        Good:
-
-            return storage.has_client(client_key) and client_key != self.dummy_client
+                return Client.exists(client_key, access_token)
         """
         raise NotImplementedError("Subclasses must implement this function.")
 
@@ -582,17 +650,22 @@ class Server(object):
         Note that if the dummy request_token is supplied it should validate in
         the same nearly the same amount of time as a valid one.
 
-        Bad:
+        Ensure latency inducing tasks are mimiced even for dummy clients.
+        For example, use::
 
-            if request_token == self.dummy_request_token:
+            from your_datastore import RequestToken
+            try:
+                return RequestToken.exists(client_key, access_token)
+            except DoesNotExist:
+                return False
+
+        Rather than::
+
+            from your_datastore import RequestToken
+            if access_token == self.dummy_access_token:
                 return False
             else:
-                return storage.has_request_token(request_token)
-
-        Good:
-
-            return (storage.has_request_token(request_token) and
-                    request_token != self.dummy_request_token)
+                return RequestToken.exists(client_key, access_token)
         """
         raise NotImplementedError("Subclasses must implement this function.")
 
@@ -602,17 +675,22 @@ class Server(object):
         Note that if the dummy access token is supplied it should validate in
         the same or nearly the same amount of time as a valid one.
 
-        Bad:
+        Ensure latency inducing tasks are mimiced even for dummy clients.
+        For example, use::
 
+            from your_datastore import AccessToken
+            try:
+                return AccessToken.exists(client_key, access_token)
+            except DoesNotExist:
+                return False
+
+        Rather than::
+
+            from your_datastore import AccessToken
             if access_token == self.dummy_access_token:
                 return False
             else:
-                return storage.has_access_token(access_token)
-
-        Good:
-
-            return (storage.has_access_token(access_token) and
-                    access_token != self.dummy_access_token)
+                return AccessToken.exists(client_key, access_token)
         """
         raise NotImplementedError("Subclasses must implement this function.")
 
@@ -630,6 +708,24 @@ class Server(object):
 
         .. _`Section 3.3`: http://tools.ietf.org/html/rfc5849#section-3.3
 
+        One of the first validation checks that will be made is for the validity
+        of the nonce and timestamp, which are associated with a client key and
+        possibly a token. If invalid then immediately fail the request
+        by returning False. If the nonce/timestamp pair has been used before and
+        you may just have detected a replay attack. Therefore it is an essential
+        part of OAuth security that you not allow nonce/timestamp reuse.
+        Note that this validation check is done before checking the validity of
+        the client and token.::
+
+           nonces_and_timestamps_database = [
+              (u'foo', 1234567890, u'rannoMstrInghere', u'bar')
+           ]
+
+           def validate_timestamp_and_nonce(self, client_key, timestamp, nonce,
+              request_token=None, access_token=None):
+
+              return ((client_key, timestamp, nonce, request_token or access_token)
+                       in self.nonces_and_timestamps_database)
         """
         raise NotImplementedError("Subclasses must implement this function.")
 
@@ -686,6 +782,16 @@ class Server(object):
         obtain token credentials and the provider must verify that the
         verifier is valid and associated with the client as well as the
         resource owner.
+
+        Verifier validation should be done in near constant time
+        (to avoid verifier enumeration). To achieve this we need a
+        constant time string comparison which is provided by OAuthLib
+        in ``oauthlib.common.safe_string_equals``::
+
+            from your_datastore import Verifier
+            correct_verifier = Verifier.get(client_key, request_token)
+            from oauthlib.common import safe_string_equals
+            return safe_string_equals(verifier, correct_verifier)
         """
         raise NotImplementedError("Subclasses must implement this function.")
 
@@ -753,7 +859,7 @@ class Server(object):
         if self.enforce_ssl and not request.uri.lower().startswith("https://"):
             raise ValueError("Insecure transport, only HTTPS is allowed.")
 
-        signature_type, params, oauth_params = self.get_signature_type_and_params(request)
+        signature_type, params, oauth_params = self._get_signature_type_and_params(request)
 
         # The server SHOULD return a 400 (Bad Request) status code when
         # receiving a request with duplicated protocol parameters.
@@ -995,13 +1101,12 @@ class Server(object):
         # prevents malicious users from guessing sensitive information
         v = all((valid_client, valid_resource_owner, valid_realm,
                     valid_redirect, valid_verifier, valid_signature))
-        logger = logging.getLogger("oauthlib")
         if not v:
-            logger.info("[Failure] OAuthLib request verification failed.")
-            logger.info("Valid client:\t%s" % valid_client)
-            logger.info("Valid token:\t%s\t(Required: %s" % (valid_resource_owner, require_resource_owner))
-            logger.info("Valid realm:\t%s\t(Required: %s)" % (valid_realm, require_realm))
-            logger.info("Valid callback:\t%s" % valid_redirect)
-            logger.info("Valid verifier:\t%s\t(Required: %s)" % (valid_verifier, require_verifier))
-            logger.info("Valid signature:\t%s" % valid_signature)
+            log.info("[Failure] OAuthLib request verification failed.")
+            log.info("Valid client:\t%s" % valid_client)
+            log.info("Valid token:\t%s\t(Required: %s" % (valid_resource_owner, require_resource_owner))
+            log.info("Valid realm:\t%s\t(Required: %s)" % (valid_realm, require_realm))
+            log.info("Valid callback:\t%s" % valid_redirect)
+            log.info("Valid verifier:\t%s\t(Required: %s)" % (valid_verifier, require_verifier))
+            log.info("Valid signature:\t%s" % valid_signature)
         return v, request
