@@ -43,6 +43,14 @@ class ClientTest(TestCase):
         client = Client(self.client_id, token_type="invalid")
         self.assertRaises(ValueError, client.add_token, self.uri)
 
+        # Case-insensitive token type
+        client = Client(self.client_id, access_token=self.access_token, token_type="bEAreR")
+        uri, headers, body = client.add_token(self.uri, body=self.body,
+                headers=self.headers)
+        self.assertURLEqual(uri, self.uri)
+        self.assertFormBodyEqual(body, self.body)
+        self.assertEqual(headers, self.bearer_header)
+
         # Missing access token
         client = Client(self.client_id)
         self.assertRaises(ValueError, client.add_token, self.uri)
@@ -215,10 +223,9 @@ class WebApplicationClientTest(TestCase):
         uri = client.prepare_request_uri(self.uri, state=self.state)
         self.assertURLEqual(uri, self.uri_state)
 
-        # With extra parameters through kwargs, checking using len since order
-        # of dict items is undefined
+        # With extra parameters through kwargs
         uri = client.prepare_request_uri(self.uri, **self.kwargs)
-        self.assertEqual(len(uri), len(self.uri_kwargs))
+        self.assertURLEqual(uri, self.uri_kwargs)
 
     def test_request_body(self):
         client = WebApplicationClient(self.client_id, code=self.code)
@@ -235,10 +242,9 @@ class WebApplicationClientTest(TestCase):
         body = client.prepare_request_body(body=self.body, redirect_uri=self.redirect_uri)
         self.assertFormBodyEqual(body, self.body_redirect)
 
-        # With extra parameters, checked using length since order of
-        # dict items is undefined
+        # With extra parameters
         body = client.prepare_request_body(body=self.body, **self.kwargs)
-        self.assertEqual(len(body), len(self.body_kwargs))
+        self.assertFormBodyEqual(body, self.body_kwargs)
 
     def test_parse_grant_uri_response(self):
         client = WebApplicationClient(self.client_id)
@@ -320,10 +326,9 @@ class UserAgentClientTest(TestCase):
         uri = client.prepare_request_uri(self.uri, state=self.state)
         self.assertURLEqual(uri, self.uri_state)
 
-        # With extra parameters through kwargs, checking using len since order
-        # of dict items is undefined
+        # With extra parameters through kwargs
         uri = client.prepare_request_uri(self.uri, **self.kwargs)
-        self.assertEqual(len(uri), len(self.uri_kwargs))
+        self.assertURLEqual(uri, self.uri_kwargs)
 
     def test_parse_token_response(self):
         client = UserAgentClient(self.client_id)
@@ -371,21 +376,20 @@ class PasswordCredentialsClientTest(TestCase):
     }
 
     def test_request_body(self):
-        client = PasswordCredentialsClient(self.client_id, self.username,
-                    self.password)
+        client = PasswordCredentialsClient(self.client_id)
 
         # Basic, no extra arguments
-        body = client.prepare_request_body(body=self.body)
+        body = client.prepare_request_body(self.username, self.password,
+                body=self.body)
         self.assertFormBodyEqual(body, self.body_up)
 
-        # With extra parameters, checked using length since order of
-        # dict items is undefined
-        body = client.prepare_request_body(body=self.body, **self.kwargs)
-        self.assertEqual(len(body), len(self.body_kwargs))
+        # With extra parameters
+        body = client.prepare_request_body(self.username, self.password,
+                body=self.body, **self.kwargs)
+        self.assertFormBodyEqual(body, self.body_kwargs)
 
     def test_parse_token_response(self):
-        client = PasswordCredentialsClient(self.client_id, self.username,
-                self.password)
+        client = PasswordCredentialsClient(self.client_id)
 
         # Parse code and state
         response = client.parse_request_body_response(self.token_json, scope=self.scope)
@@ -436,10 +440,9 @@ class ClientCredentialsClientTest(TestCase):
         body = rclient.prepare_request_body(body=self.body)
         self.assertFormBodyEqual(body, self.body_up)
 
-        # With extra parameters, checked using length since order of
-        # dict items is undefined
+        # With extra parameters
         body = client.prepare_request_body(body=self.body, **self.kwargs)
-        self.assertEqual(len(body), len(self.body_kwargs))
+        self.assertFormBodyEqual(body, self.body_kwargs)
 
     def test_parse_token_response(self):
         client = ClientCredentialsClient(self.client_id)
