@@ -24,8 +24,8 @@ class CommonTests(TestCase):
         self.assertItemsEqual(urldecode('c2='), [('c2', '')])
         self.assertItemsEqual(urldecode('foo=bar'), [('foo', 'bar')])
         self.assertItemsEqual(urldecode('foo_%20~=.bar-'), [('foo_ ~', '.bar-')])
-        self.assertItemsEqual(urldecode('foo bar'), [('foo bar', '')])
-        self.assertItemsEqual(urldecode('?'), [('?', '')])
+        self.assertRaises(ValueError, urldecode, 'foo bar')
+        self.assertRaises(ValueError, urldecode, '?')
         self.assertRaises(ValueError, urldecode, '%R')
         self.assertRaises(ValueError, urldecode, '%RA')
         self.assertRaises(ValueError, urldecode, '%AR')
@@ -47,7 +47,7 @@ class CommonTests(TestCase):
         self.assertItemsEqual(extract_params([]), [])
 
     def test_extract_non_formencoded_string(self):
-        self.assertEqual(extract_params('not a formencoded string'), [('not a formencoded string', '')])
+        self.assertEqual(extract_params('not a formencoded string'), None)
 
     def test_extract_invalid(self):
         self.assertEqual(extract_params(object()), None)
@@ -82,7 +82,7 @@ class CommonTests(TestCase):
     def test_non_formencoded_string_body(self):
         body = 'foo bar'
         r = Request(self.uri, body=body)
-        self.assertEqual(r.decoded_body, [('foo bar', '')])
+        self.assertEqual(r.decoded_body, None)
 
     def test_param_free_sequence_body(self):
         body = [1, 1, 2, 3, 5, 8, 13]
@@ -121,3 +121,25 @@ class CommonTests(TestCase):
         token = generate_token(length=6, chars="python")
         self.assertEqual(len(token), 6)
         self.assertNotIn("a", token)
+
+    def test_generate_client_id(self):
+        client_id = generate_client_id()
+        self.assertEqual(len(client_id), 30)
+
+        client_id = generate_client_id(length=44)
+        self.assertEqual(len(client_id), 44)
+
+        client_id = generate_client_id(length=6, chars="python")
+        self.assertEqual(len(client_id), 6)
+        self.assertNotIn("a", client_id)
+
+
+class CaseInsensitiveDictTest(TestCase):
+
+    def test_basic(self):
+        cid = CaseInsensitiveDict({})
+        cid['a'] = 'b'
+        cid['c'] = 'd'
+        del cid['c']
+        self.assertEqual(cid['A'], 'b')
+        self.assertEqual(cid['a'], 'b')
